/**********************************************************
 
FreeHandBatch.jsx

DESCRIPTION

For use in Illustrator CS6 or later with the Tensai FreeHand Plugin (version 2.2 or later) installed
Open all Freehand files specified in the user selected folder and save them as AI files
Version 4.0

History:
Version 1.0 Initial version (22 March 2015)
Version 2.0 Add Keep Missing Font setting (14 October 2015)
Version 3.0 Support for file names without extension (20 January 2017)
Version 4.0 Add character encoding setting and fit to view (30 May 2017)
Version 5.0 Add choice of replacement font and overset text scaling and replace displayName with name (21 May 2021)
**********************************************************/

// FreeHand import options
var ConvertTextToOutlines = false;
var ImportSinglePage = false;
var ImportPageNumber = 1;
var KeepMissingFonts = true; // (Plugin version 2.3.0 or later)
var CharacterEncoding = 0; // (Plugin version 2.5.0 or later) 1 = Chinese, 2 = Japanese, 3 = Hebrew
var ReplacementFont = "Myriad Pro"; // (Plugin version 4.0.0 or later)
var AdjustOverset = false; // (Plugin version 4.0.0 or later)
var OversetScaling = 95; // (Plugin version 4.0.0 or later)
var FitToView = true;
// End of FreeHand import options

try
{
	// Get the folder to read files from
	var inputFolderPath = null;
	inputFolderPath = Folder.selectDialog( 'Select Input Folder containing Freehand Files.', '~' );
    
    if (inputFolderPath != null) {
        // Parse the folder name to get Folder Name Prefix
        var inputFolderPrefix = inputFolderPath.fullName;

        // Get the folder to save the files into
        var destFolder = null;
        destFolder = Folder.selectDialog( 'Select Output Folder for Illustrator files.', '~' );

        
        app.userInteractionLevel = UserInteractionLevel.DONTDISPLAYALERTS;

        // Set Freehand options
		app.preferences.setBooleanPreference("fhi_Text2Outlines",ConvertTextToOutlines);
		app.preferences.setBooleanPreference("fhi_ImportSinglePage",ImportSinglePage);
		app.preferences.setIntegerPreference("fhi_PageNumber",ImportPageNumber);
		app.preferences.setBooleanPreference("fhi_KeepMissingFonts",KeepMissingFonts);
        app.preferences.setIntegerPreference("fhi_Encoding",CharacterEncoding);
        app.preferences.removePreference("fhi_ReplacementFont");
        app.preferences.setStringPreference("fhi_ReplacementFont",ReplacementFont);
        app.preferences.setBooleanPreference("fhi_AdjustOverset",AdjustOverset);
        app.preferences.setIntegerPreference("fhi_OversetScaling",OversetScaling);

        // Recursively read all Freehand files and convert to AI
        ConvertFHToAI(inputFolderPath);
    }
}
catch (err)
{
   rs = ("ERROR: " + (err.number & 0xFFFF) + ", " + err.description);
   alert(rs);
}


function ConvertFHToAI(inputFolderPath)
{
  
	var testFiles = new Array();
	var index = 0;
    
	// Create output folder path to be created
	pathToAppend = inputFolderPath.fullName.split(inputFolderPrefix);
	var saveFolder = destFolder + "/" + pathToAppend[1];

	// Create folder
    
	fldr = new Folder(saveFolder);
	fldr.create();
    
    if (!(fldr.exists))
    {
        throw new Error('Access is denied');
    }    
    
	// Get list of files/folders in the current working folder
	testFiles = inputFolderPath.getFiles("*.*");

	for (index = 0; index < testFiles.length;index++)
	{
        // Check if current item is file or folder
        if (testFiles[index] instanceof Folder)
        {
            ConvertFHToAI(testFiles[index]);
        }
        else
        {
            // Selected Item is a file
            var fileName = testFiles[index].name;
            
            var fileExtensionArray = fileName.split(".", 2);
			var parts = fileExtensionArray.length;

            // Check is file is a freehand file
            var isFreeHand = false;
			if (parts > 1)
			{
				var fileExtension = fileExtensionArray[parts-1].toUpperCase();
            
                isFreeHand = (fileExtension == "FH11" || fileExtension == "FH10" || fileExtension == "FH9" || fileExtension == "FH8" || fileExtension == "FH7" );
            }
            else
            {
                testFiles[index].open('r');
                testFiles[index].encoding = "BINARY";
                var header = testFiles[index].read(4);
                isFreeHand = ((header.substring(0,3) == 'AGD') && (header[3] >= '2') && (header[3] <= '5'));
                if (!isFreeHand)
                {
                    testFiles[index].seek(26,0);
                    header = testFiles[index].read(10);
                    isFreeHand = ((header.substring(0,9) == 'FreeHand1') && (header[9] >= '0') && (header[9] <= '1'));
                }
            }
            if (isFreeHand)
            {
                // Open Freehand file
                var docRef = app.open(testFiles[index]);
                 
				// Fit to view
				if (FitToView && app.executeMenuCommand)
					app.executeMenuCommand('fitall');

                // Create output file path
                var filePreName = fileName.split(".",1);
                sDocumentPath = saveFolder +"/"+ filePreName + ".ai" ;
		
                // Save file as AI
                SaveAsAI(docRef,sDocumentPath);
                    
                // Close the document
                docRef.close( SaveOptions.DONOTSAVECHANGES );
            }
        }
    }   
} 

function SaveAsAI(docRef,sDocumentPath)
{
    theFile = new File(sDocumentPath);
                    
    // Create AI Save options
    var aiOptions  = new IllustratorSaveOptions();
    
    aiOptions.compressed = true;
    aiOptions.embedICCProfile = false;
    aiOptions.embedLinkedFiles = false;
    aiOptions.flattenOutput = OutputFlattening.PRESERVEAPPEARANCE;
    aiOptions.fontSubsetThreshold = 100;
    aiOptions.pdfCompatible = true;
    docRef.saveAs (theFile, aiOptions);
}
